@@ -4,10 +4,12 @@
 
 """aiocoap-client is a simple command-line tool for interacting with CoAP servers"""
 
+import copy
 import sys
 import asyncio
 import argparse
 import logging
+import signal
 import subprocess
 from pathlib import Path
 
@@ -28,9 +30,74 @@ from aiocoap.util import contenttype
 from aiocoap.util.cli import ActionNoYes
 from aiocoap.numbers import ContentFormat
 
+log = logging.getLogger("coap.aiocoap-client")
 
-def build_parser():
-    p = argparse.ArgumentParser(description=__doc__)
+
+def augment_parser_for_global(p, *, prescreen=False):
+    p.add_argument(
+        "-v",
+        "--verbose",
+        help="Increase the debug output",
+        action="count",
+    )
+    p.add_argument(
+        "-q",
+        "--quiet",
+        help="Decrease the debug output",
+        action="count",
+    )
+    p.add_argument(
+        "--version", action="version", version="%(prog)s " + aiocoap.meta.version
+    )
+
+    p.add_argument(
+        "--interactive",
+        help="Enter interactive mode. Combine with --help or run `help` interactively to see which options apply where; some can be used globally and overwritten locally.",
+        action="store_true",
+    )
+
+
+def augment_parser_for_either(p):
+    p.add_argument(
+        "--color",
+        help="Color output (default on TTYs if all required modules are installed)",
+        default=None,
+        action=ActionNoYes,
+    )
+    p.add_argument(
+        "--pretty-print",
+        help="Pretty-print known content formats (default on TTYs if all required modules are installed)",
+        default=None,
+        action=ActionNoYes,
+    )
+    p.add_argument(
+        "--proxy", help="Relay the CoAP request to a proxy for execution", metavar="URI"
+    )
+    p.add_argument(
+        "--credentials",
+        help="Load credentials to use from a given file",
+        type=Path,
+    )
+    p.add_argument(
+        "--no-set-hostname",
+        help="Suppress transmission of Uri-Host even if the host name is not an IP literal",
+        dest="set_hostname",
+        action="store_false",
+        default=True,
+    )
+    p.add_argument(
+        "--no-sec",
+        # Can be "Send request without any security" once it actually does
+        # anything; until then, it's fine as a no-op to not impede changing
+        # scripts later.
+        help=argparse.SUPPRESS,
+        dest="sec",
+        action="store_false",
+        default=None,
+    )
+
+
+def augment_parser_for_interactive(p, *, prescreen=False):
     p.add_argument(
         "--non",
         help="Send request as non-confirmable (NON) message",
@@ -55,9 +122,6 @@ def build_parser():
         help="Content format to request",
         metavar="MIME",
     )
-    p.add_argument(
-        "--proxy", help="Relay the CoAP request to a proxy for execution", metavar="URI"
-    )
     p.add_argument(
         "--payload",
         help="Send X as request payload (eg. with a PUT). If X starts with an '@', its remainder is treated as a file name and read from; '@-' reads from the console. Non-file data may be recoded, see --content-format.",
@@ -74,62 +138,36 @@ def build_parser():
         help="Content format of the --payload data. If a known format is given and --payload has a non-file argument, the payload is converted from CBOR Diagnostic Notation.",
         metavar="MIME",
     )
-    p.add_argument(
-        "--no-set-hostname",
-        help="Suppress transmission of Uri-Host even if the host name is not an IP literal",
-        dest="set_hostname",
-        action="store_false",
-        default=True,
-    )
-    p.add_argument(
-        "-v",
-        "--verbose",
-        help="Increase the debug output",
-        action="count",
-    )
-    p.add_argument(
-        "-q",
-        "--quiet",
-        help="Decrease the debug output",
-        action="count",
-    )
-    # careful: picked before parsing
-    p.add_argument(
-        "--interactive",
-        help="Enter interactive mode",
-        action="store_true",
-    )
-    p.add_argument(
-        "--credentials",
-        help="Load credentials to use from a given file",
-        type=Path,
-    )
-    p.add_argument(
-        "--version", action="version", version="%(prog)s " + aiocoap.meta.version
-    )
-
-    p.add_argument(
-        "--color",
-        help="Color output (default on TTYs if all required modules are installed)",
-        default=None,
-        action=ActionNoYes,
-    )
-    p.add_argument(
-        "--pretty-print",
-        help="Pretty-print known content formats (default on TTYs if all required modules are installed)",
-        default=None,
-        action=ActionNoYes,
-    )
     p.add_argument(
         "url",
+        nargs="?" if prescreen else None,
         help="CoAP address to fetch",
     )
 
+
+def build_parser(*, use_global=True, use_interactive=True, prescreen=False):
+    p = argparse.ArgumentParser(description=__doc__, add_help=not prescreen)
+    if prescreen:
+        p.add_argument("--help", action="store_true")
+    if use_global:
+        augment_parser_for_global(p, prescreen=prescreen)
+    augment_parser_for_either(p)
+    if use_interactive:
+        augment_parser_for_interactive(p, prescreen=prescreen)
+
     return p
 
 
-def configure_logging(verbosity):
-    logging.basicConfig()
+def configure_logging(verbosity, color):
+    if color is not False:
+        try:
+            import colorlog
+        except ImportError:
+            color = False
+        else:
+            colorlog.basicConfig()
+    if not color:
+        logging.basicConfig()
 
     if verbosity <= -2:
         logging.getLogger("coap").setLevel(logging.CRITICAL + 1)
@@ -138,9 +176,16 @@ def configure_logging(verbosity):
     elif verbosity == 0:
         logging.getLogger("coap").setLevel(logging.WARNING)
     elif verbosity == 1:
+        logging.getLogger("coap").setLevel(logging.WARNING)
+        logging.getLogger("coap.aiocoap-client").setLevel(logging.INFO)
+    elif verbosity == 2:
         logging.getLogger("coap").setLevel(logging.INFO)
-    elif verbosity >= 2:
+    elif verbosity >= 3:
         logging.getLogger("coap").setLevel(logging.DEBUG)
+    elif verbosity >= 4:
+        logging.getLogger("coap").setLevel(0)
+
+    log.debug("Logging configured.")
 
 
 def colored(text, options, tokenlambda):
@@ -161,6 +206,10 @@ def colored(text, options, tokenlambda):
 
 
 def incoming_observation(options, response):
+    log.info("Received Observe notification:")
+    for line in message_to_text(response, "from"):
+        log.info(line)
+
     if options.observe_exec:
         p = subprocess.Popen(options.observe_exec, shell=True, stdin=subprocess.PIPE)
         # FIXME this blocks
@@ -182,25 +231,60 @@ def incoming_observation(options, response):
 
 
 def apply_credentials(context, credentials, errfn):
-    if credentials.suffix == ".json":
-        import json
+    try:
+        if credentials.suffix == ".json":
+            import json
 
-        context.client_credentials.load_from_dict(json.load(credentials.open("rb")))
-    elif credentials.suffix == ".diag":
-        try:
-            import cbor_diag
-            import cbor2
-        except ImportError:
-            raise errfn(
-                "Loading credentials in CBOR diagnostic format requires cbor2 and cbor_diag package"
+            context.client_credentials.load_from_dict(json.load(credentials.open("rb")))
+        elif credentials.suffix == ".diag":
+            try:
+                import cbor_diag
+                import cbor2
+            except ImportError:
+                raise errfn(
+                    "Loading credentials in CBOR diagnostic format requires cbor2 and cbor_diag package"
+                )
+            context.client_credentials.load_from_dict(
+                cbor2.loads(cbor_diag.diag2cbor(credentials.open().read()))
             )
-        context.client_credentials.load_from_dict(
-            cbor2.loads(cbor_diag.diag2cbor(credentials.open().read()))
-        )
+        else:
+            raise errfn(
+                "Unknown suffix: %s (expected: .json or .diag)" % (credentials.suffix)
+            )
+    except FileNotFoundError as e:
+        raise errfn("Credential file not found: %s" % e.filename)
+    except (OSError, ValueError) as e:
+        # Any of the parsers could reasonably raise those, and while they don't
+        # have HelpfulError support, they should still not render a backtrace
+        # but a proper CLI error.
+        raise errfn("Processing credential file: %s" % e)
+
+
+def message_to_text(m, direction):
+    """Convert a message to a text form similar to how they are shown in RFCs.
+
+    Refactoring this into a message method will need to address the direction
+    discovery eventually."""
+    if m.remote is None:
+        # This happens when unprocessable remotes are, eg. putting in an HTTP URI
+        yield f"{m.code} {direction} (unknown)"
     else:
-        raise errfn(
-            "Unknown suffix: %s (expected: .json or .diag)" % (credentials.suffix)
-        )
+        # FIXME: Update when transport-indication is available
+        # FIXME: This is slightly wrong because it does not account for what ProxyRedirector does
+        yield f"{m.code} {direction} {m.remote.scheme}://{m.remote.hostinfo}"
+    for opt in m.opt.option_list():
+        if hasattr(opt.number, "name"):
+            yield f"- {opt.number.name_printable} ({opt.number.value}): {opt.value!r}"
+        else:
+            yield f"- {opt.number.value}: {opt.value!r}"
+    if m.payload:
+        limit = 16
+        if len(m.payload) > limit:
+            yield f"Payload: {m.payload[:limit].hex()}... ({len(m.payload)} bytes total)"
+        else:
+            yield f"Payload: {m.payload[:limit].hex()} ({len(m.payload)} bytes)"
+    else:
+        yield "No payload"
 
 
 def present(message, options, file=sys.stdout):
@@ -286,9 +370,9 @@ def present(message, options, file=sys.stdout):
                 file.write("\n")
 
 
-async def single_request(args, context):
-    parser = build_parser()
-    options = parser.parse_args(args)
+async def single_request(args, context, globalopts=None):
+    parser = build_parser(use_global=globalopts is None)
+    options = parser.parse_args(args, copy.copy(globalopts))
 
     pretty_print_modules = aiocoap.defaults.prettyprint_missing_modules()
     if pretty_print_modules and (options.color is True or options.pretty_print is True):
@@ -302,8 +386,6 @@ async def single_request(args, context):
     if options.pretty_print is None:
         options.pretty_print = sys.stdout.isatty() and not pretty_print_modules
 
-    configure_logging((options.verbose or 0) - (options.quiet or 0))
-
     try:
         try:
             code = getattr(
@@ -400,13 +482,17 @@ async def single_request(args, context):
         if options.payload_initial_szx is not None:
             request.remote.maximum_block_size_exp = options.payload_initial_szx
 
-        if options.proxy is None:
+        if options.proxy is None or options.proxy in ("none", "", "-"):
             interface = context
         else:
             interface = aiocoap.proxy.client.ProxyForwarder(options.proxy, context)
 
         requested_uri = request.get_request_uri()
 
+        log.info("Sending request:")
+        for line in message_to_text(request, "to"):
+            log.info(line)
+
         requester = interface.request(request)
 
         if options.observe:
@@ -423,6 +509,10 @@ async def single_request(args, context):
             if options.observe and not requester.observation.cancelled:
                 requester.observation.cancel()
 
+        log.info("Received response:")
+        for line in message_to_text(response_data, "from"):
+            log.info(line)
+
         response_uri = response_data.get_request_uri()
         if requested_uri != response_uri:
             print(
@@ -493,7 +583,7 @@ async def single_request_with_context(args):
 interactive_expecting_keyboard_interrupt = None
 
 
-async def interactive():
+async def interactive(globalopts):
     global interactive_expecting_keyboard_interrupt
     interactive_expecting_keyboard_interrupt = asyncio.get_event_loop().create_future()
 
@@ -515,8 +605,20 @@ async def interactive():
         if line in (["quit"], ["q"], ["exit"]):
             break
 
+        async def single_request_noexit(*args, **kwargs):
+            """Protects a run against the exit automatically generated by
+            argparse errors or help"""
+            try:
+                # We could also set exit_on_error, but this way we do get the
+                # original error code and can show that; might still revisit.
+                await single_request(*args, **kwargs)
+            except SystemExit as e:
+                return e.code
+            else:
+                return 0
+
         current_task = asyncio.create_task(
-            single_request(line, context=context),
+            single_request_noexit(line, context=context, globalopts=globalopts),
             name="Interactive prompt command %r" % line,
         )
         interactive_expecting_keyboard_interrupt = (
@@ -532,49 +634,54 @@ async def interactive():
             current_task.cancel()
         else:
             try:
-                await current_task
-            except SystemExit as e:
-                if e.code != 0:
-                    print("Exit code: %d" % e.code, file=sys.stderr)
-                continue
+                code = await current_task
             except Exception as e:
                 print("Unhandled exception raised: %s" % (e,))
+            if code != 0:
+                print("Exit code: %d" % code, file=sys.stderr)
 
     await context.shutdown()
 
 
-def sync_main(args=None):
+async def main(args=None):
     # interactive mode is a little messy, that's why this is not using aiocoap.util.cli yet
     if args is None:
         args = sys.argv[1:]
 
-    if "--interactive" not in args:
+    # This one is tolerant and doesn't even terminate with --help, so that
+    # --help and --interactive --help can do the right thing.
+    first_parser = build_parser(prescreen=True)
+    first_args = first_parser.parse_args(args)
+
+    configure_logging(
+        (first_args.verbose or 0) - (first_args.quiet or 0), first_args.color
+    )
+
+    if not first_args.interactive:
         try:
-            asyncio.run(single_request_with_context(args))
-        except KeyboardInterrupt:
+            await single_request_with_context(args)
+        except asyncio.exceptions.CancelledError:
             sys.exit(3)
     else:
-        if len(args) != 1:
-            print(
-                "No other arguments must be specified when entering interactive mode",
-                file=sys.stderr,
-            )
-            sys.exit(1)
+        global_parser = build_parser(use_interactive=False)
+        globalopts = global_parser.parse_args(args)
 
         loop = asyncio.get_event_loop()
-        task = loop.create_task(
-            interactive(),
-            name="Interactive prompt",
-        )
 
-        while not task.done():
+        def ctrl_c():
             try:
-                loop.run_until_complete(task)
-            except KeyboardInterrupt:
-                if not interactive_expecting_keyboard_interrupt.done():
-                    interactive_expecting_keyboard_interrupt.set_result(None)
-            except SystemExit:
-                continue  # asyncio/tasks.py(242) raises those after setting them as results, but we particularly want them back in the loop
+                interactive_expecting_keyboard_interrupt.set_result(None)
+            except asyncio.exceptions.InvalidStateError:
+                # Too many Ctlr-C before the program could clean up
+                sys.exit(3)
+
+        loop.add_signal_handler(signal.SIGINT, ctrl_c)
+
+        await interactive(globalopts)
+
+
+def sync_main(args=None):
+    asyncio.run(main(args=args))
 
 
 if __name__ == "__main__":
