@@ -8,7 +8,7 @@ Known Caveats:
 
     * It is very permissive. Not only is no security implemented.
 
-    * This may and will make exotic choices about discoverable paths whereever
+    * This may and will make exotic choices about discoverable paths wherever
       it can (see StandaloneResourceDirectory documentation)
 
     * Split-horizon is not implemented correctly
@@ -299,7 +299,7 @@ class CommonRD:
 
     def _new_pathtail(self):
         for i in itertools.count(1):
-            # In the spirit of making legal but unconvential choices (see
+            # In the spirit of making legal but unconventional choices (see
             # StandaloneResourceDirectory documentation): Whoever strips or
             # ignores trailing slashes shall have a hard time keeping
             # registrations alive.
@@ -369,7 +369,7 @@ class CommonRD:
 
         # this was the brutal way towards idempotency (delete and re-create).
         # if any actions based on that are implemented here, they have yet to
-        # decide wheter they'll treat idempotent recreations like deletions or
+        # decide whether they'll treat idempotent recreations like deletions or
         # just ignore them unless something otherwise unchangeable (ep, d)
         # changes.
 
@@ -820,35 +820,39 @@ def build_parser():
 
     add_server_arguments(p)
 
+    p.add_argument(
+        "--version", action="version", version="%(prog)s " + aiocoap.meta.version
+    )
+    p.add_argument(
+        "--proxy-domain",
+        help="Enable the RD proxy extension. Example: `proxy.example.net` will produce base URIs like `coap://node1.proxy.example.net/`. The names must all resolve to an address the RD is bound to.",
+        type=str,
+    )
+    p.add_argument(
+        "--lwm2m-compat",
+        help="Compatibility mode for LwM2M clients that can not perform some discovery steps (moving the registration resource to `/rd`)",
+        action="store_true",
+        default=None,
+    )
+    p.add_argument(
+        "--no-lwm2m-compat",
+        help="Disable all compatibility with LwM2M clients that can not perform some discovery steps (not even accepting registrations at `/rd` with warnings)",
+        action="store_false",
+        dest="lwm2m_compat",
+    )
+    p.add_argument(
+        "--verbose",
+        help="Increase debug log output (repeat for increased verbosity)",
+        action="count",
+        default=0,
+    )
+
     return p
 
 
 class Main(AsyncCLIDaemon):
     async def start(self, args=None):
         parser = build_parser()
-        parser.add_argument(
-            "--proxy-domain",
-            help="Enable the RD proxy extension. Example: `proxy.example.net` will produce base URIs like `coap://node1.proxy.example.net/`. The names must all resolve to an address the RD is bound to.",
-            type=str,
-        )
-        parser.add_argument(
-            "--lwm2m-compat",
-            help="Compatibility mode for LwM2M clients that can not perform some discovery steps (moving the registration resource to `/rd`)",
-            action="store_true",
-            default=None,
-        )
-        parser.add_argument(
-            "--no-lwm2m-compat",
-            help="Disable all compativility with LwM2M clients that can not perform some discovery steps (not even accepting registrations at `/rd` with warnings)",
-            action="store_false",
-            dest="lwm2m_compat",
-        )
-        parser.add_argument(
-            "--verbose",
-            help="Increase debug log output (repeat for increased verbosity)",
-            action="count",
-            default=0,
-        )
         options = parser.parse_args(args if args is not None else sys.argv[1:])
 
         # Putting in an empty site to construct the site with a context
