@@ -10,6 +10,7 @@ import warnings
 import abc
 import errno
 from typing import Optional
+import ipaddress
 
 from .numbers import codes
 from . import util
@@ -33,11 +34,11 @@ class HelpfulError(Error):
         error message itself may be insufficient to point the user in the right
         direction
 
-        The `hints` dictonary may be populated with context that the caller
+        The `hints` dictionary may be populated with context that the caller
         has; the implementation must tolerate their absence. Currently
         established keys:
 
-        * original_uri (str): URI that was attemted to access
+        * original_uri (str): URI that was attempted to access
         * request (Message): Request that was assembled to be sent
         """
         return None
@@ -263,13 +264,37 @@ class NetworkError(HelpfulError):
                 # seen trying to reach any unused local address
                 return "No way of contacting the remote host could be found. This could be because a host on the local network is offline or firewalled. Tools for debugging in the next step could be ping or traceroute."
             if self.__cause__.errno == errno.ENETUNREACH:
-                # seen trying to reach an IPv6 host through an IP literal from a v4-only system, or trying to reach 2001:db8::1
-                return "No way of contacting the remote network could be found. This may be due to lack of IPv6 connectivity, lack of a concrete route (eg. trying to reach a private use network which there is no route to). Tools for debugging in the next step could be ping or traceroute."
+                connectivity_text = ""
+                try:
+                    host, _ = util.hostportsplit(hints["request"].remote.hostinfo)
+                    ip = ipaddress.ip_address(host)
+                    if isinstance(ip, ipaddress.IPv4Address):
+                        connectivity_text = "IPv4 "
+                    if isinstance(ip, ipaddress.IPv6Address):
+                        connectivity_text = "IPv6 "
+
+                    # Else, we can't help because we don't have access to the
+                    # name resolution result. This could all still be enhanced
+                    # by threading along the information somewhere.
+                except Exception:
+                    # This is not the point to complain about bugs, just give a more generic error.
+                    pass
+
+                # seen trying to reach an IPv6 host through an IP literal from a v4-only system, or trying to reach 2001:db8::1, or 192.168.254.254
+                return f"No way of contacting the remote network could be found. This may be due to lack of {connectivity_text}connectivity, or lack of a concrete route (eg. trying to reach a private use network which there is no route to). Tools for debugging in the next step could be ping or traceroute."
             if self.__cause__.errno == errno.EACCES:
                 # seen trying to reach the broadcast address of a local network
                 return "The operating system refused to send the request. For example, this can occur when attempting to send broadcast requests instead of multicast requests."
 
 
+class NoRequestInterface(RuntimeError, ConstructionRenderableError, NetworkError):
+    code = codes.PROXYING_NOT_SUPPORTED
+    message = "Error: No CoAP transport available for this scheme on any request interface."  # or address, but we don't take transport-indication in full yet
+
+    def extra_help(self, hints={}):
+        return "More transports may be enabled by installing extra optional dependencies. Alternatively, consider using a CoAP-CoAP proxy."
+
+
 class ResolutionError(NetworkError):
     """Resolving the host component of a URI to a usable transport address was
     not possible"""
@@ -333,6 +358,22 @@ class WaitingForClientTimedOut(TimeoutError):
     """
 
 
+class ConToMulticast(ValueError, HelpfulError):
+    # This could become a RenderableError if a proxy gains the capability to
+    # influence whether or not a message is sent reliably strongly (ie. beyond
+    # hints), and reliable transport is required in a request sent to
+    # multicast.
+    """
+    Raised when attempting to send a confirmable message to a multicast address.
+    """
+
+    def __str__(self):
+        return "Refusing to send CON message to multicast address"
+
+    def extra_help(self, hints={}):
+        return "Requests over UDP can only be sent to unicast addresses as per RFC7252; pick a concrete peer or configure a NON request instead."
+
+
 class ResourceChanged(Error):
     """
     The requested resource was modified during the request and could therefore
