@@ -62,7 +62,7 @@ class EndpointAddress(metaclass=abc.ABCMeta):
     property of a message can mean the message is exchanged "with
     [2001:db8::2:1]:5683, while my local address was [2001:db8:1::1]:5683"
     (typical of UDP6), "over the connected <Socket at
-    0x1234>, whereever that's connected to" (simple6 or TCP) or "with
+    0x1234>, wherever that's connected to" (simple6 or TCP) or "with
     participant 0x01 of the OSCAP key 0x..., routed over <another
     EndpointAddress>".
 
@@ -309,11 +309,11 @@ class RequestProvider(metaclass=abc.ABCMeta):
 
 class Request(metaclass=abc.ABCMeta):
     """A CoAP request, initiated by sending a message. Typically, this is not
-    instanciated directly, but generated by a :meth:`RequestProvider.request`
+    instantiated directly, but generated by a :meth:`RequestProvider.request`
     method."""
 
     response = """A future that is present from the creation of the object and \
-        fullfilled with the response message.
+        fulfilled with the response message.
 
         When legitimate errors occur, this becomes an aiocoap.Error. (Eg. on
         any kind of network failure, encryption trouble, or protocol
@@ -378,7 +378,7 @@ class Resource(metaclass=abc.ABCMeta):
         if await self.needs_blockwise_assembly(req):
             req = self._block1.feed_and_take(req)
 
-            # Note that unless the lambda get's called, we're not fully
+            # Note that unless the lambda gets called, we're not fully
             # accessing req any more -- we're just looking at its block2
             # option, and the blockwise key extracted earlier.
             res = await self._block2.extract_or_insert(req, lambda: self.render(req))
