@@ -37,6 +37,14 @@ an operator:
 * ERROR is used when something clearly went wrong. This includes irregular
   connection terminations and resource handler errors (which are demoted to
   error responses), and can often contain a backtrace.
+
+Logs will generally reveal messages exchanged between this and other systems,
+and attackers can observe their encrypted counterparts. Private or shared keys
+are only logged through an internal `log_secret` function, which usually
+replaces them with a redacted value. Setting the ``AIOCOAP_REVEAL_KEYS``
+environment variable to the value ``show secrets in logs`` bypasses that
+mechanism. As an additional precaution, this is only accepted if the effective
+user has write access to the aiocoap source code.
 """
 
 import asyncio
@@ -129,7 +137,7 @@ class Context(interfaces.RequestProvider):
         self.server_credentials = server_credentials or CredentialsMap()
 
     #
-    # convenience methods for class instanciation
+    # convenience methods for class instantiation
     #
 
     async def _append_tokenmanaged_messagemanaged_transport(
@@ -457,7 +465,7 @@ class Context(interfaces.RequestProvider):
         for ri in self.request_interfaces:
             if await ri.fill_or_recognize_remote(message):
                 return ri
-        raise RuntimeError("No request interface could route message")
+        raise error.NoRequestInterface()
 
     def request(self, request_message, handle_blockwise=True):
         if handle_blockwise:
@@ -508,7 +516,7 @@ class Context(interfaces.RequestProvider):
         return await self.serversite.render_to_pipe(pipe)
 
 
-class BaseRequest(object):
+class BaseRequest:
     """Common mechanisms of :class:`Request` and :class:`MulticastRequest`"""
 
 
@@ -796,12 +804,12 @@ class BlockwiseRequest(BaseUnicastRequest, interfaces.Request):
                 DeprecationWarning,
                 stacklevel=2,
             )
-            assert (
-                app_request.opt.block1.block_number == 0
-            ), "Unexpected block number in app_request"
-            assert (
-                not app_request.opt.block1.more
-            ), "Unexpected more-flag in app_request"
+            assert app_request.opt.block1.block_number == 0, (
+                "Unexpected block number in app_request"
+            )
+            assert not app_request.opt.block1.more, (
+                "Unexpected more-flag in app_request"
+            )
             # this is where the library user can traditionally pass in size
             # exponent hints into the library.
             size_exp = app_request.opt.block1.size_exponent
@@ -904,7 +912,7 @@ class BlockwiseRequest(BaseUnicastRequest, interfaces.Request):
                 if not blockresponse.code.is_successful():
                     break
                 else:
-                    # ignoring (discarding) the successul intermediate result, waiting for a final one
+                    # ignoring (discarding) the successful intermediate result, waiting for a final one
                     continue
 
         lower_observation = None
@@ -1133,7 +1141,7 @@ class ClientObservation:
         interface instead."""
         if not _suppress_deprecation:
             warnings.warn(
-                "register_callback on observe results is deprected: Use `async for notify in request.observation` instead.",
+                "register_callback on observe results is deprecated: Use `async for notify in request.observation` instead.",
                 DeprecationWarning,
                 stacklevel=2,
             )
@@ -1153,7 +1161,7 @@ class ClientObservation:
         interface instead."""
         if not _suppress_deprecation:
             warnings.warn(
-                "register_errback on observe results is deprected: Use `async for notify in request.observation` instead.",
+                "register_errback on observe results is deprecated: Use `async for notify in request.observation` instead.",
                 DeprecationWarning,
                 stacklevel=2,
             )
