@@ -19,7 +19,7 @@ membership management and ``recvmsg`` to obtain data configured with the above
 options. The need for ``AI_V4MAPPED`` and ``AI_ADDRCONFIG`` is not manifest
 in the code because the latter on its own is insufficient to enable seamless
 interoperability with IPv4+IPv6 servers on IPv4-only hosts; instead,
-short-lived sockets are crated to assess which addresses are routable. This
+short-lived sockets are created to assess which addresses are routable. This
 should correctly deal with situations in which a client has an IPv6 ULA
 assigned but no route, no matter whether the server advertises global IPv6
 addresses or addresses inside that ULA. It can not deal with situations in
@@ -50,7 +50,7 @@ import struct
 import weakref
 from collections import namedtuple
 
-from ..message import Message
+from ..message import Message, Direction
 from ..numbers import constants
 from .. import defaults
 from .. import error
@@ -151,7 +151,7 @@ class UDP6EndpointAddress(interfaces.EndpointAddress):
         return str(address)
 
     def _plainaddress(self):
-        """Return the IP adress part of the sockaddr in IPv4 notation if it is
+        """Return the IP address part of the sockaddr in IPv4 notation if it is
         mapped, otherwise the plain v6 address including the interface
         identifier if set."""
 
@@ -269,7 +269,7 @@ class MessageInterfaceUDP6(RecvmsgDatagramProtocol, interfaces.MessageInterface)
             None  #: Future created and used in the .shutdown() method.
         )
 
-        self.ready = asyncio.get_running_loop().create_future()  #: Future that gets fullfilled by connection_made (ie. don't send before this is done; handled by ``create_..._context``
+        self.ready = asyncio.get_running_loop().create_future()  #: Future that gets fulfilled by connection_made (ie. don't send before this is done; handled by ``create_..._context``
 
         # This is set while a send is underway to determine in the
         # error_received call site whom we were actually sending something to.
@@ -294,7 +294,7 @@ class MessageInterfaceUDP6(RecvmsgDatagramProtocol, interfaces.MessageInterface)
         cls, sock, ctx: interfaces.MessageManager, log, loop, multicast=[]
     ):
         try:
-            sock.setsockopt(socket.IPPROTO_IPV6, socknumbers.IPV6_RECVPKTINFO, 1)
+            sock.setsockopt(socket.IPPROTO_IPV6, socket.IPV6_RECVPKTINFO, 1)
         except NameError:
             raise RuntimeError(
                 "RFC3542 PKTINFO flags are unavailable, unable to create a udp6 transport."
@@ -430,11 +430,11 @@ class MessageInterfaceUDP6(RecvmsgDatagramProtocol, interfaces.MessageInterface)
         ancdata = []
         if message.remote.pktinfo is not None:
             ancdata.append(
-                (socket.IPPROTO_IPV6, socknumbers.IPV6_PKTINFO, message.remote.pktinfo)
+                (socket.IPPROTO_IPV6, socket.IPV6_PKTINFO, message.remote.pktinfo)
             )
-        assert (
-            self._remote_being_sent_to.get(None) is None
-        ), "udp6.MessageInterfaceUDP6.send was reentered in a single task"
+        assert self._remote_being_sent_to.get(None) is None, (
+            "udp6.MessageInterfaceUDP6.send was reentered in a single task"
+        )
         self._remote_being_sent_to.set(message.remote)
         try:
             self.transport.sendmsg(
@@ -547,10 +547,7 @@ class MessageInterfaceUDP6(RecvmsgDatagramProtocol, interfaces.MessageInterface)
         """Implementation of the RecvmsgDatagramProtocol interface, called by the transport."""
         pktinfo = None
         for cmsg_level, cmsg_type, cmsg_data in ancdata:
-            if (
-                cmsg_level == socket.IPPROTO_IPV6
-                and cmsg_type == socknumbers.IPV6_PKTINFO
-            ):
+            if cmsg_level == socket.IPPROTO_IPV6 and cmsg_type == socket.IPV6_PKTINFO:
                 pktinfo = cmsg_data
             else:
                 self.log.info(
@@ -570,6 +567,7 @@ class MessageInterfaceUDP6(RecvmsgDatagramProtocol, interfaces.MessageInterface)
         except error.UnparsableMessage:
             self.log.warning("Ignoring unparsable message from %s", address)
             return
+        message.direction = Direction.INCOMING
 
         try:
             self._ctx.dispatch_message(message)
@@ -583,23 +581,20 @@ class MessageInterfaceUDP6(RecvmsgDatagramProtocol, interfaces.MessageInterface)
             raise
 
     def datagram_errqueue_received(self, data, ancdata, flags, address):
-        assert (
-            flags == socknumbers.MSG_ERRQUEUE
-        ), "Received non-error data through the errqueue"
+        assert flags == socknumbers.MSG_ERRQUEUE, (
+            "Received non-error data through the errqueue"
+        )
         pktinfo = None
         errno_value = None
         for cmsg_level, cmsg_type, cmsg_data in ancdata:
-            assert (
-                cmsg_level == socket.IPPROTO_IPV6
-            ), "Received non-IPv6 protocol through the errqueue"
+            assert cmsg_level == socket.IPPROTO_IPV6, (
+                "Received non-IPv6 protocol through the errqueue"
+            )
             if cmsg_type == socknumbers.IPV6_RECVERR:
                 extended_err = SockExtendedErr.load(cmsg_data)
-                self.log.debug("Socket error recevied, details: %s", extended_err)
+                self.log.debug("Socket error received, details: %s", extended_err)
                 errno_value = extended_err.ee_errno
-            elif (
-                cmsg_level == socket.IPPROTO_IPV6
-                and cmsg_type == socknumbers.IPV6_PKTINFO
-            ):
+            elif cmsg_level == socket.IPPROTO_IPV6 and cmsg_type == socket.IPV6_PKTINFO:
                 pktinfo = cmsg_data
             else:
                 self.log.info(
