@@ -21,9 +21,9 @@ class ActionNoYes(argparse.Action):
     # https://stackoverflow.com/questions/9234258/in-python-argparse-is-it-possible-to-have-paired-no-something-something-arg#9236426
     def __init__(self, option_strings, dest, default=True, required=False, help=None):
         assert len(option_strings) == 1, "ActionNoYes takes only one option name"
-        assert option_strings[0].startswith(
-            "--"
-        ), "ActionNoYes options must start with --"
+        assert option_strings[0].startswith("--"), (
+            "ActionNoYes options must start with --"
+        )
         super().__init__(
             ["--" + option_strings[0][2:], "--no-" + option_strings[0][2:]],
             dest,
@@ -117,7 +117,7 @@ class AsyncCLIDaemon:
             # <https://unix.stackexchange.com/questions/10231/when-does-the-system-send-a-sigterm-to-a-process>)
             exitcode = await main.__exitcode
         except KeyboardInterrupt:
-            logging.info("Keyboard interupt received, shutting down")
+            logging.info("Keyboard interrupt received, shutting down")
             sys.exit(3)
         else:
             sys.exit(exitcode)
